-- Copyright 1999-2012. Parallels IP Holdings GmbH. All Rights Reserved.

--
-- Table structure for table `aps_resource_adjacency_list`
--

/*!40101 SET @saved_cs_client     = @@character_set_client */;
/*!40101 SET character_set_client = utf8 */;
CREATE TABLE IF NOT EXISTS `aps_resource_adjacency_list` (
  `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `base_resource_id` int(10) unsigned NOT NULL,
  `dependent_resource_id` int(10) unsigned NOT NULL,
  `relation` char(1) NOT NULL DEFAULT 'e',
  `uniqueness_scope` varchar(255) DEFAULT NULL,
  `branch` varchar(255) DEFAULT NULL,
  `requirement_id` varchar(255) NOT NULL DEFAULT '',
  PRIMARY KEY (`id`),
  UNIQUE KEY `IX_aps_resource_adjacency_list_unique_link` (`base_resource_id`,`dependent_resource_id`),
  KEY `FK_aps_resource_adjacency_list_base` (`base_resource_id`),
  KEY `FK_aps_resource_adjacency_list_dependent` (`dependent_resource_id`),
  CONSTRAINT `FK_aps_resource_adjacency_list_base` FOREIGN KEY (`base_resource_id`) REFERENCES `aps_resource` (`id`) ON DELETE NO ACTION ON UPDATE NO ACTION,
  CONSTRAINT `FK_aps_resource_adjacency_list_dependent` FOREIGN KEY (`dependent_resource_id`) REFERENCES `aps_resource` (`id`) ON DELETE NO ACTION ON UPDATE NO ACTION
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
/*!40101 SET character_set_client = @saved_cs_client */;

--
-- Table structure for table `aps_resource_usage_report`
--

/*!40101 SET @saved_cs_client     = @@character_set_client */;
/*!40101 SET character_set_client = utf8 */;
CREATE TABLE IF NOT EXISTS `aps_resource_usage_report` (
  `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `resource_id` int(10) unsigned NOT NULL,
  `last_update_timestamp` bigint(20) unsigned NOT NULL,
  `poll_interval` int(10) unsigned NOT NULL DEFAULT '0',
  PRIMARY KEY (`id`),
  UNIQUE KEY `IX_aps_resource_usage_report_resource` (`resource_id`),
  CONSTRAINT `FK_aps_resource_usage_report_aps_resource` FOREIGN KEY (`resource_id`) REFERENCES `aps_resource` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
/*!40101 SET character_set_client = @saved_cs_client */;

--
-- Table structure for table `aps_resource_usage_counter`
--

/*!40101 SET @saved_cs_client     = @@character_set_client */;
/*!40101 SET character_set_client = utf8 */;
CREATE TABLE IF NOT EXISTS `aps_resource_usage_counter` (
  `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `report_id` int(10) unsigned NOT NULL,
  `counter_id` varchar(64) NOT NULL,
  `unit` varchar(32) NOT NULL,
  `value` int(10) unsigned NOT NULL DEFAULT '0',
  `limit_value` bigint(20) unsigned NOT NULL DEFAULT '0',
  PRIMARY KEY (`id`),
  UNIQUE KEY `IX_aps_resource_usage_counter_report` (`report_id`,`counter_id`),
  CONSTRAINT `FK_aps_resource_usage_counter_aps_resource_usage_report` FOREIGN KEY (`report_id`) REFERENCES `aps_resource_usage_report` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;
/*!40101 SET character_set_client = @saved_cs_client */;

SET @apsDefaultSchema = '@@APSC_DATABASE@@';
SET @saved_cs_client     = @@character_set_client;
SET character_set_client = utf8;

-- Add 'app_id' column to 'aps_resource' table.
DROP PROCEDURE IF EXISTS aps_add_resource_app_id;

DELIMITER //
CREATE PROCEDURE aps_add_resource_app_id()
BEGIN
IF NOT EXISTS (SELECT * FROM `information_schema`.`COLUMNS` 
	WHERE `COLUMN_NAME` = 'app_id' 
	AND `TABLE_NAME` = 'aps_resource'
	AND `TABLE_SCHEMA` = CONVERT(@apsDefaultSchema USING utf8))
THEN
ALTER TABLE `aps_resource` ADD COLUMN `app_id` int(10) unsigned DEFAULT NULL;
ALTER TABLE `aps_resource` ADD KEY `FK_aps_resource_aps_application` (`app_id`);
ALTER TABLE `aps_resource` ADD CONSTRAINT `FK_aps_resource_aps_application` FOREIGN KEY (`app_id`) REFERENCES `aps_application` (`id`) 
 ON DELETE NO ACTION ON UPDATE NO ACTION;
END IF;
END//
DELIMITER ;

CALL aps_add_resource_app_id();

DROP PROCEDURE IF EXISTS aps_add_resource_app_id;

UPDATE `aps_resource` SET `app_id` = (SELECT `ar`.`app_id` FROM `aps_application_resource` `ar` WHERE `ar`.`resource_id` = `id`);

-- Registering parent resources in adjacency list.
DELETE FROM `aps_resource_adjacency_list` WHERE `relation` = 'p';
INSERT INTO `aps_resource_adjacency_list` (`base_resource_id`, `dependent_resource_id`, `relation`, `requirement_id`)
  SELECT `parent_resource_id`, `id`, 'p', '' FROM `aps_resource` WHERE `parent_resource_id` IS NOT NULL;

-- Registering resources for required objects.
DROP PROCEDURE IF EXISTS aps_register_required_resources;

DELIMITER //
CREATE PROCEDURE aps_register_required_resources()
BEGIN
DECLARE tmp_req_cur CURSOR FOR SELECT `uid`, `type` FROM `apstmp_requirement_object`;
OPEN tmp_req_cur;
BEGIN
DECLARE tmp_req_uid varchar(36);
DECLARE tmp_req_type varchar(255);
DECLARE EXIT HANDLER FOR NOT FOUND BEGIN END;
LOOP
FETCH tmp_req_cur INTO tmp_req_uid, tmp_req_type;
INSERT INTO `aps_registry_object` (`uid`, `type`, `enabled`) VALUES (tmp_req_uid, 'aps.resource', 'y');
INSERT INTO `aps_resource` (`registry_object_id`, `service_id`, `resource_type`) VALUES (LAST_INSERT_ID(), '', tmp_req_type);
END LOOP;
END;
CLOSE tmp_req_cur;
END//
DELIMITER ;

DROP TEMPORARY TABLE IF EXISTS `apstmp_requirement_object`;
CREATE TEMPORARY TABLE `apstmp_requirement_object` (`uid` varchar(36) NOT NULL, `type` varchar(255) NOT NULL)
  SELECT DISTINCT(`rr`.`uid`), `rr`.`type` FROM `aps_resource_requirement` `rr`
  LEFT JOIN `aps_registry_object` `ro` ON `ro`.`uid` = `rr`.`uid` WHERE `ro`.`id` IS NULL;

CALL aps_register_required_resources();

DROP PROCEDURE IF EXISTS aps_register_required_resources;

-- Registering resource requirements in adjacency list.
DROP PROCEDURE IF EXISTS aps_update_requirement_adjacency_list;

DELIMITER //
CREATE PROCEDURE aps_update_requirement_adjacency_list()
BEGIN
DECLARE tmp_req_cur CURSOR FOR SELECT `id`, `resource_id`, `requirement_id`, `branch` FROM `apstmp_requirement_resource`;
OPEN tmp_req_cur;
BEGIN
DECLARE tmp_req_id int(10) unsigned;
DECLARE tmp_req_res_id int(10) unsigned;
DECLARE tmp_req_req_id varchar(255);
DECLARE tmp_req_branch varchar(255);
DECLARE EXIT HANDLER FOR NOT FOUND BEGIN END;
LOOP
FETCH tmp_req_cur INTO tmp_req_id, tmp_req_res_id, tmp_req_req_id, tmp_req_branch;
INSERT INTO `aps_resource_adjacency_list` (`base_resource_id`, `dependent_resource_id`, `relation`, `requirement_id`, `branch`)
 VALUES (tmp_req_id, tmp_req_res_id, 'e', tmp_req_req_id, tmp_req_branch);
END LOOP;
END;
END//
DELIMITER ;

DROP TEMPORARY TABLE IF EXISTS `apstmp_requirement_resource`;
CREATE TEMPORARY TABLE `apstmp_requirement_resource` (
  `id` int(10) unsigned NOT NULL,
  `resource_id` int(10) unsigned NOT NULL,
  `requirement_id` varchar(255) NOT NULL,
  `branch` varchar(255) default NULL
) SELECT `r`.`id`, `rr`.`resource_id`, `rr`.`requirement_id`, `rr`.`branch` FROM `aps_resource_requirement` `rr` 
  INNER JOIN `aps_registry_object` `ro` ON `ro`.`uid` = `rr`.`uid`
  INNER JOIN `aps_resource` `r` ON `r`.`registry_object_id` = `ro`.`id`
  LEFT JOIN `aps_resource_adjacency_list` `ral` ON `ral`.`base_resource_id` = `r`.`id`
    AND `ral`.`dependent_resource_id` = `rr`.`resource_id` WHERE `ral`.`id` IS NULL;

CALL aps_update_requirement_adjacency_list();

DROP PROCEDURE IF EXISTS aps_update_requirement_adjacency_list;

-- Drop obsolete tables
DROP TABLE IF EXISTS `aps_application_resource`;

CREATE TABLE IF NOT EXISTS `aps_resource_log` (
  `id` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `resource_id` int(10) unsigned NOT NULL,
  `uid` varchar(36) NOT NULL,
  `severity` int(10) unsigned NOT NULL,
  `content` varchar(4000) NOT NULL,
  `log_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
  PRIMARY KEY (`id`),
  KEY `FK_aps_resource_log_resource_id` (`resource_id`),
  CONSTRAINT `FK_aps_resource_log_resource_id` FOREIGN KEY (`resource_id`) REFERENCES `aps_resource` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;


CREATE TABLE IF NOT EXISTS `aps_log_param` (
  `resource_log_id` int(10) unsigned NOT NULL,
  `name` varchar(64) NOT NULL,
  `value` varchar(4000) NOT NULL,
  PRIMARY KEY (`resource_log_id`,`name`),
  CONSTRAINT `FK_aps_log_param_resource_log_id` FOREIGN KEY (`resource_log_id`) REFERENCES `aps_resource_log` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8;

SET character_set_client = @saved_cs_client;
